@@ -28,6 +28,7 @@ and the Primer3 library.
 #include    <stdio.h>
 #include    <Python.h>
 #include    <libprimer3.h>
+#include    "p3_seq_lib.h"
 
 
 #if PY_MAJOR_VERSION < 3
@@ -114,13 +115,13 @@ and the Primer3 library.
                     return -1;}                                                \
                 memcpy(*st, tc, (int)(ss + 1));                                \
             }                                                                  \
-        }                                                                      
+        }
 #else
     #define DICT_GET_AND_COPY_STR(o, d, k, st, tc, ss)                         \
         if (DICT_GET_OBJ(o, d, k)) {                                           \
             int from_int = 0;                                                  \
             if (PyUnicode_Check(o)) {                                          \
-                tc = PyUnicode_AsUTF8AndSize(o, &ss);                          \
+                tc = (char *)PyUnicode_AsUTF8AndSize(o, &ss);                  \
             } else if (PyBytes_Check(o)){                                      \
                 if (PyBytes_AsStringAndSize(o, &tc, &ss) == -1) {              \
                     return -1;}                                                \
@@ -264,7 +265,7 @@ pdh_setGlobals(p3_global_settings *pa, PyObject *p3s_dict) {
     Py_ssize_t              str_size;
     char                    *temp_char=NULL, *task_tmp=NULL;
 
-    
+
     // if (!(pa = p3_create_global_settings())) {
     //     PyErr_SetString(PyExc_IOError, "Could not allocate memory for p3 globals");
     //     return NULL;
@@ -315,7 +316,7 @@ pdh_setGlobals(p3_global_settings *pa, PyObject *p3s_dict) {
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PAIR_MAX_COMPL_ANY", pa->pair_compl_any);
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PAIR_MAX_COMPL_END", pa->pair_compl_end);
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PAIR_MAX_COMPL_ANY_TH", pa->pair_compl_any_th);
-    DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PAIR_MAX_COMPL_END_TH", pa->p_args.divalent_conc);
+    DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PAIR_MAX_COMPL_END_TH", pa->pair_compl_end_th);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_PICK_ANYWAY", pa->pick_anyway);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_GC_CLAMP", pa->gc_clamp);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_MAX_END_GC", pa->max_end_gc);
@@ -333,10 +334,10 @@ pdh_setGlobals(p3_global_settings *pa, PyObject *p3s_dict) {
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PRODUCT_MAX_TM", pa->product_max_tm);
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PRODUCT_MIN_TM", pa->product_min_tm);
     DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_PRODUCT_OPT_TM", pa->product_opt_tm);
-    DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_SEQUENCING_LEAD", pa->sequencing.lead);
-    DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_SEQUENCING_SPACING", pa->sequencing.spacing);
-    DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_SEQUENCING_INTERVAL", pa->sequencing.interval);
-    DICT_GET_AND_ASSIGN_DOUBLE(p_obj, p3s_dict, "PRIMER_SEQUENCING_ACCURACY", pa->sequencing.accuracy);
+    DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_SEQUENCING_LEAD", pa->sequencing.lead);
+    DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_SEQUENCING_SPACING", pa->sequencing.spacing);
+    DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_SEQUENCING_INTERVAL", pa->sequencing.interval);
+    DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_SEQUENCING_ACCURACY", pa->sequencing.accuracy);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_MIN_5_PRIME_OVERLAP_OF_JUNCTION", pa->min_5_prime_overlap_of_junction);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_MIN_3_PRIME_OVERLAP_OF_JUNCTION", pa->min_3_prime_overlap_of_junction);
     DICT_GET_AND_ASSIGN_INT(p_obj, p3s_dict, "PRIMER_PICK_RIGHT_PRIMER", pa->pick_right_primer);
@@ -449,14 +450,14 @@ pdh_setGlobals(p3_global_settings *pa, PyObject *p3s_dict) {
                     flat_list = 1;
                 } else {
                     PyErr_Format(PyExc_TypeError,\
-                        "\"PRIMER_PRODUCT_SIZE_RANGE\" contains mixed sequence objects and integers"); 
+                        "\"PRIMER_PRODUCT_SIZE_RANGE\" contains mixed sequence objects and integers");
                     Py_DECREF(p_obj2);
-                    Py_DECREF(p_obj3);  
-                    return -1;                 
+                    Py_DECREF(p_obj3);
+                    return -1;
                 }
             }
             Py_DECREF(p_obj2);
-            Py_DECREF(p_obj3); 
+            Py_DECREF(p_obj3);
         }
         if (!flat_list) {
             for (i=0; i < product_size_range_list_length; i++){
@@ -493,6 +494,7 @@ pdh_setGlobals(p3_global_settings *pa, PyObject *p3s_dict) {
                 Py_DECREF(p_obj3);
                 Py_DECREF(p_obj4);
             }
+            pa->num_intervals = i;
         }
     }
 
@@ -588,13 +590,13 @@ pdh_createSeqLib(PyObject *seq_dict){
                     "Cannot add seq with non-String type to seq_lib");
                 goto err_create_seq_lib;
             }
-            if (add_seq_and_rev_comp_to_seq_lib(sl, seq, seq_name, errfrag)) {
+            if (add_seq_to_seq_lib(sl, seq, seq_name, errfrag)) {
                 PyErr_SetString(PyExc_IOError, errfrag);
                 goto err_create_seq_lib;
             }
 #else
             if (PyUnicode_Check(py_seq_name)) {
-                seq_name = PyUnicode_AsUTF8(py_seq_name);
+                seq_name = (char *)PyUnicode_AsUTF8(py_seq_name);
             } else if (PyBytes_Check(py_seq_name)){
                 seq_name = PyBytes_AsString(py_seq_name);
             } else {
@@ -603,7 +605,7 @@ pdh_createSeqLib(PyObject *seq_dict){
                 goto err_create_seq_lib;
             }
             if (PyUnicode_Check(py_seq)) {
-                seq = PyUnicode_AsUTF8(py_seq);
+                seq = (char *)PyUnicode_AsUTF8(py_seq);
             } else if (PyBytes_Check(py_seq)){
                 seq = PyBytes_AsString(py_seq);
             } else {
@@ -611,12 +613,13 @@ pdh_createSeqLib(PyObject *seq_dict){
                     "Cannot add seq with non-Unicode/Bytes type to seq_lib");
                 goto err_create_seq_lib;
             }
-            if (add_seq_and_rev_comp_to_seq_lib(sl, seq, seq_name, errfrag)) {
+            if (add_seq_to_seq_lib(sl, seq, seq_name, errfrag)) {
                 PyErr_SetString(PyExc_IOError, errfrag);
                 goto err_create_seq_lib;
             }
 #endif
     }
+    reverse_complement_seq_lib(sl);
     return sl;
 err_create_seq_lib:
     destroy_seq_lib(sl);
@@ -647,7 +650,7 @@ pdh_setSeqArgs(PyObject *sa_dict, seq_args *sa) {
     if (DICT_GET_OBJ(p_obj, sa_dict, "SEQUENCE_PRIMER_PAIR_OK_REGION_LIST")){
         int ii[4], flat_list = 0;
         if (!PySequence_Check(p_obj)){
-            PyErr_SetString(PyExc_IOError, "Value of 'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' " 
+            PyErr_SetString(PyExc_IOError, "Value of 'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' "
                             "must support the seqeunce protocol.");
             return -1;
         }
@@ -689,7 +692,7 @@ pdh_setSeqArgs(PyObject *sa_dict, seq_args *sa) {
                     return -1;
                 }
                 len2 = (int)PySequence_Size(p_obj2);
-                if (!len2 == 4) {
+                if (!(len2 == 4)) {
                     PyErr_Format(PyExc_TypeError, "Sub-list/tuple #%d of "
                                  "'SEQUENCE_PRIMER_PAIR_OK_REGION_LIST' must "
                                  "of length 4", i);
@@ -716,7 +719,7 @@ pdh_setSeqArgs(PyObject *sa_dict, seq_args *sa) {
     DICT_GET_AND_COPY_TO_INTERVAL_ARRAY(p_obj, sa_dict, "SEQUENCE_EXCLUDED_REGION", sa->excl2);
     DICT_GET_AND_COPY_TO_INTERVAL_ARRAY(p_obj, sa_dict, "SEQUENCE_INTERNAL_EXCLUDED_REGION", sa->excl_internal2);
     // DICT_GET_AND_COPY_ARRAY_INTO_ARRAY(p_obj, sa_dict,  "SEQUENCE_OVERLAP_JUNCTION_LIST", &sa->primer_overlap_junctions, overlap_junction_len);
-    
+
     if (DICT_GET_OBJ(p_obj, sa_dict, "SEQUENCE_OVERLAP_JUNCTION_LIST")) {
         int *poj_arr, single_value=0;
         PyObject *arr_item;
@@ -768,7 +771,7 @@ pdh_setSeqArgs(PyObject *sa_dict, seq_args *sa) {
                     "\"SEQUENCE_INCLUDED_REGION\" contains non-int value");
                 Py_DECREF(seq_item1);
                 Py_DECREF(seq_item2);
-                return -1;                
+                return -1;
             }
             sa->incl_s = (int)PyLong_AsLong(seq_item1);
             sa->incl_l = (int)PyLong_AsLong(seq_item2);
@@ -895,6 +898,41 @@ pdh_setSeqArgs(PyObject *sa_dict, seq_args *sa) {
         SET_DICT_KEY_TO_OBJ(dict, key, obj_ptr)
 #endif
 
+#if PY_MAJOR_VERSION < 3
+    #define SET_DICT_KEY_TO_TUPLE_OF_FLOAT_AND_STR(dict, key, num, str, obj_ptr)\
+        if ((obj_ptr = PyTuple_New(2)) == NULL) {                              \
+            PyErr_Format(PyExc_IOError, "Could not create tuple for %s ", key);\
+            return NULL;                                                       \
+        }                                                                      \
+        if (PyTuple_SetItem(obj_ptr, 0, PyFloat_FromDouble(num))) {            \
+            PyErr_Format(PyExc_IOError,                                        \
+                         "Could not pack value 1 for %s into tuple", key);     \
+            return NULL;                                                       \
+        }                                                                      \
+        if (PyTuple_SetItem(obj_ptr, 1, PyString_FromString(str))) {           \
+            PyErr_Format(PyExc_IOError,                                        \
+                         "Could not pack value 2 for %s into tuple",key);      \
+            return NULL;                                                       \
+        }                                                                      \
+        SET_DICT_KEY_TO_OBJ(dict, key, obj_ptr)
+#else
+    #define SET_DICT_KEY_TO_TUPLE_OF_FLOAT_AND_STR(dict, key, num, str, obj_ptr)\
+        if ((obj_ptr = PyTuple_New(2)) == NULL) {                              \
+            PyErr_Format(PyExc_IOError, "Could not create tuple for %s ", key);\
+            return NULL;                                                       \
+        }                                                                      \
+        if (PyTuple_SetItem(obj_ptr, 0, PyFloat_FromDouble(num))) {            \
+            PyErr_Format(PyExc_IOError,                                        \
+                         "Could not pack value 1 for %s into tuple", key);     \
+            return NULL;                                                       \
+        }                                                                      \
+        if (PyTuple_SetItem(obj_ptr, 1, PyUnicode_FromString(str))) {          \
+            PyErr_Format(PyExc_IOError,                                        \
+                         "Could not pack value 2 for %s into tuple",key);      \
+            return NULL;                                                       \
+        }                                                                      \
+        SET_DICT_KEY_TO_OBJ(dict, key, obj_ptr)
+#endif
 
 PyObject*
 pdh_outputToDict(const p3_global_settings *pa, const seq_args *sa,
@@ -1006,7 +1044,7 @@ pdh_outputToDict(const p3_global_settings *pa, const seq_args *sa,
     }
     if (pa->pick_right_primer == 1
       && pa->pick_left_primer == 1) {
-        SET_DICT_KEY_TO_STR(output_dict, "PRIMER_INTERNAL_EXPLAIN", \
+        SET_DICT_KEY_TO_STR(output_dict, "PRIMER_PAIR_EXPLAIN", \
             p3_get_pair_array_explain_string(p3_get_rv_best_pairs(retval)), \
             obj_ptr);
     }
@@ -1285,19 +1323,19 @@ pdh_outputToDict(const p3_global_settings *pa, const seq_args *sa,
     if (seq_lib_num_seq(pa->p_args.repeat_lib) > 0) {
         if (go_fwd == 1){
             sprintf(outbuff, "PRIMER_LEFT%s_LIBRARY_MISPRIMING", suffix);
-            SET_DICT_KEY_TO_TUPLE_OF_LONG_AND_STR(output_dict, outbuff, \
+            SET_DICT_KEY_TO_TUPLE_OF_FLOAT_AND_STR(output_dict, outbuff, \
                 fwd->repeat_sim.score[fwd->repeat_sim.max], \
                 fwd->repeat_sim.name, obj_ptr);
         }
         if (go_rev == 1){
             sprintf(outbuff, "PRIMER_RIGHT%s_LIBRARY_MISPRIMING", suffix);
-            SET_DICT_KEY_TO_TUPLE_OF_LONG_AND_STR(output_dict, outbuff, \
+            SET_DICT_KEY_TO_TUPLE_OF_FLOAT_AND_STR(output_dict, outbuff, \
                 rev->repeat_sim.score[rev->repeat_sim.max], \
                 rev->repeat_sim.name, obj_ptr);
         }
         if (retval->output_type == primer_pairs){
             sprintf(outbuff, "PRIMER_PAIR%s_LIBRARY_MISPRIMING", suffix);
-            SET_DICT_KEY_TO_TUPLE_OF_LONG_AND_STR(output_dict, outbuff, \
+            SET_DICT_KEY_TO_TUPLE_OF_FLOAT_AND_STR(output_dict, outbuff, \
                 retval->best_pairs.pairs[i].repeat_sim, \
                 retval->best_pairs.pairs[i].rep_name, obj_ptr);
         }
@@ -1305,7 +1343,7 @@ pdh_outputToDict(const p3_global_settings *pa, const seq_args *sa,
     /* Print out internal oligo mispriming scores */
     if (go_int == 1 && seq_lib_num_seq(pa->o_args.repeat_lib) > 0) {
         sprintf(outbuff, "PRIMER_%s%s_LIBRARY_MISPRIMING", int_oligo, suffix);
-        SET_DICT_KEY_TO_TUPLE_OF_LONG_AND_STR(output_dict, outbuff, \
+        SET_DICT_KEY_TO_TUPLE_OF_FLOAT_AND_STR(output_dict, outbuff, \
             intl->repeat_sim.score[intl->repeat_sim.max], \
             intl->repeat_sim.name, obj_ptr);
     }
