@@ -1,4 +1,4 @@
-# Copyright (C) 2014. Ben Pruitt & Nick Conway; Wyss Institute
+# Copyright (C) 2014-2020. Ben Pruitt & Nick Conway; Wyss Institute
 # See LICENSE for full GPLv2 license.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -98,25 +98,35 @@ _ntthal_re = re.compile(b'dS\s+=\s+(\S+)\s+dH\s+=\s+(\S+)\s+' +
                         b'dG\s+=\s+(\S+)\s+t\s+=\s+(\S+)')
 
 THERMORESULT = namedtuple('thermoresult', [
-    'result',           # True if a structure is present
-    'ds',               # Entropy (cal/(K*mol))
-    'dh',               # Enthalpy (kcal/mol)
-    'dg',               # Gibbs free energy
-    'tm']               # Melting temperature (deg. Celsius)
+        'result',           # True if a structure is present
+        'ds',               # Entropy (cal/(K*mol))
+        'dh',               # Enthalpy (kcal/mol)
+        'dg',               # Gibbs free energy
+        'tm',               # Melting temperature (deg. Celsius)
+        'ascii_structure'   # ASCII representation of structure
+    ]
 )
 
-NULLTHERMORESULT = THERMORESULT(False, 0, 0, 0, 0)
+NULLTHERMORESULT = THERMORESULT(False, 0, 0, 0, 0, '')
 
 def _parse_ntthal(ntthal_output):
     ''' Helper method that uses regex to parse ntthal output. '''
     parsed_vals = re.search(_ntthal_re, ntthal_output)
-    return THERMORESULT(
-        True,                           # Structure found
-        float(parsed_vals.group(1)),    # dS
-        float(parsed_vals.group(2)),    # dH
-        float(parsed_vals.group(3)),    # dG
-        float(parsed_vals.group(4))     # tm
-    ) if parsed_vals else NULLTHERMORESULT
+    if parsed_vals:
+        ascii_structure = (
+            ntthal_output[ntthal_output.index(b'\n') + 1:].decode('utf-8')
+        )
+        res = THERMORESULT(
+            True,                           # Structure found
+            float(parsed_vals.group(1)),    # dS
+            float(parsed_vals.group(2)),    # dH
+            float(parsed_vals.group(3)),    # dG
+            float(parsed_vals.group(4)),    # tm
+            ascii_structure
+        )
+    else:
+        res = NULLTHERMORESULT
+    return res
 
 
 def calcThermo(seq1, seq2, calc_type='ANY', mv_conc=50, dv_conc=0,
